
# q-expansion of newform 126.6.g.j, downloaded from the LMFDB on 07 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 126
weight = 6
poly_data = [6241, 0, 79, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 2, 0, 0], 1], [[0, 0, 1, 0], 79], [[0, 0, 0, 2], 79]]

hecke_ring_character_values = [[29, [1, 0, 0, 0]], [73, [-1, 0, -1, 0]]]
aps_data = [[4, 0, 4, 0], [0, 0, 0, 0], [35, 4, 35, 0], [21, 0, 42, -7], [0, 7, -31, 7], [910, 0, 0, 14], [0, 22, -847, 22], [-413, -39, -413, 0], [-1367, 119, -1367, 0], [1426, 0, 0, 238], [0, 55, -1337, 55], [4573, 126, 4573, 0], [-3066, 0, 0, -42], [-8020, 0, 0, 672], [-12663, -87, -12663, 0], [0, -1050, -7479, -1050], [0, -307, 553, -307], [14021, -46, 14021, 0], [0, -469, -51321, -469], [5528, 0, 0, 812], [0, -1576, 17535, -1576], [-50881, 3269, -50881, 0], [22316, 0, 0, -4396], [-37737, 3252, -37737, 0], [-4158, 0, 0, 1078], [0, 2696, 21049, 2696], [-30905, -4659, -30905, 0], [-73131, -2709, -73131, 0], [0, -1624, -32505, -1624], [90382, 0, 0, -8582], [132736, 0, 0, -3892], [209909, 4141, 209909, 0], [0, 4130, -261887, 4130], [-76692, 0, 0, 7588], [-5133, -16212, -5133, 0], [0, 1043, -223017, 1043], [0, 20250, 145943, 20250], [109867, 9807, 109867, 0], [227920, 0, 0, 12082], [-13573, 3658, -13573, 0], [0, -357, 504081, -357], [-124586, 0, 0, 5768], [-27687, 29841, -27687, 0], [0, 5334, 45503, 5334], [281258, 0, 0, 37142], [0, -30969, 89495, -30969], [562004, 0, 0, 20244], [-71680, 0, 0, -43092], [0, -73065, 31857, -73065], [-933555, 4336, -933555, 0], [600871, -49798, 600871, 0], [-223848, 0, 0, -36666], [0, -18096, 1131263, -18096], [730716, 0, 0, -29946], [267855, -11910, 267855, 0], [0, -30597, -998847, -30597], [0, 9710, 1032745, 9710], [-1535233, -12919, -1535233, 0], [0, 56994, 470039, 56994], [135014, 0, 0, -63742], [0, -90679, 165375, -90679], [636426, 0, 0, 56448], [1125012, 0, 0, 70672], [0, 30647, 1789921, 30647], [-445095, 68512, -445095, 0], [-1050261, 88578, -1050261, 0], [-1185245, 31143, -1185245, 0], [2094178, 0, 0, -69790], [0, -65667, 1817673, -65667], [1136814, 0, 0, 97762], [0, 122634, 708897, 122634], [1490081, -89327, 1490081, 0], [0, 14937, 1882223, 14937], [278301, -252182, 278301, 0], [233172, 0, 0, 43246], [-3334247, -28843, -3334247, 0], [0, 12824, 1470361, 12824], [-3184699, -24700, -3184699, 0], [-3255681, 25746, -3255681, 0], [0, -124482, 1546559, -124482], [-3746148, 0, 0, -48048], [3157702, 0, 0, -200410], [0, -130179, -2408871, -130179], [-732046, 0, 0, 284046], [-1572529, -213553, -1572529, 0], [-184571, 253127, -184571, 0], [-2825426, 0, 0, 108430], [-4059543, 108136, -4059543, 0], [52066, 0, 0, -94346], [-7503312, 0, 0, -53984], [2935877, -266087, 2935877, 0], [0, -182741, -845551, -182741], [0, -120435, -2311201, -120435], [-660228, 0, 0, 362082], [-4981317, -249641, -4981317, 0], [2221800, 0, 0, 178836], [2164827, -347616, 2164827, 0], [0, -201396, 7898289, -201396], [1521163, -161235, 1521163, 0], [-81499, 401744, -81499, 0], [-4403348, 0, 0, 447146], [0, 635432, 929401, 635432], [0, -103235, 2647673, -103235], [-972729, -420000, -972729, 0], [0, 118489, 1608799, 118489], [0, 550976, 3243135, 550976], [-5745964, 0, 0, 324968], [-5719665, -353874, -5719665, 0], [0, 288183, -3948255, 288183], [-14962150, 0, 0, 74886], [2891399, -59917, 2891399, 0], [0, -180740, -1889801, -180740], [-612250, 0, 0, -486682], [0, 223103, 4635351, 223103], [2772632, 0, 0, -193452], [0, -184520, -1171327, -184520], [14234276, 0, 0, -92064], [0, 584655, -6736863, 584655], [-6202821, 115458, -6202821, 0], [11957804, 0, 0, -79660], [0, -402298, -2675001, -402298], [-9157678, 0, 0, -537726], [12446259, 466008, 12446259, 0], [0, 178059, 16034337, 178059], [10299835, 56787, 10299835, 0], [-2932526, 0, 0, -1245944], [-7791299, 120750, -7791299, 0], [-4790079, 518151, -4790079, 0], [13761720, 0, 0, -308756], [-4866379, -458602, -4866379, 0], [0, -764687, -3502097, -764687], [-20499432, 0, 0, 177492], [8881847, -490273, 8881847, 0], [-4651610, 0, 0, -742], [5650855, -709978, 5650855, 0], [-10081806, 0, 0, -737366], [0, -726592, -14937671, -726592], [0, 366127, 22282855, 366127], [1225266, 0, 0, -137802], [0, -487480, 19066849, -487480], [-13649748, 0, 0, 97048], [17585251, 86576, 17585251, 0], [0, 397145, -4170849, 397145], [23381412, 0, 0, 194712], [0, -1125496, 8034663, -1125496], [23702840, 0, 0, 526400], [6601910, 0, 0, 1169406], [0, -762432, -10586079, -762432], [13030843, -1505655, 13030843, 0], [12343217, 289555, 12343217, 0], [4998933, 1694560, 4998933, 0], [4332510, 0, 0, -250488], [21549908, 0, 0, 157500], [-27760047, 29061, -27760047, 0], [0, 1500835, 2362615, 1500835], [-3791664, 0, 0, 2104704], [-11169681, -1086113, -11169681, 0], [-129297, -2021484, -129297, 0], [47322506, 0, 0, -41496], [0, 1147322, 22462489, 1147322], [33460229, 550333, 33460229, 0], [15758342, 0, 0, -694918], [19243016, 0, 0, -1001364], [-25486755, -1143495, -25486755, 0], [0, -229278, 2900769, -229278], [0, -50763, 27254409, -50763], [0, 1733949, 7419023, 1733949], [0, 2439198, 331751, 2439198]]
